
import django_couch
from django.conf import settings

from django.contrib.auth.models import check_password, get_hexdigest
from django_couch.auth.models import User
from couchdbcurl.client import ResourceNotFound
    
class CouchBackend(object):

    def __init__(self):
        self.db = django_couch.db(settings.COUCHDB_AUTH_DB)

    def get_user(self, user_id):
        try:
            user = self.db[user_id]
            if len(self.db.view(settings.COUCHDB_AUTH_VIEW, key=user.username, limit=1).rows):
                return User(self.db[user_id], _db = self.db)
            else:
                return None
        except ResourceNotFound:
            return None
        

    def authenticate(self, username, password):
        rows = self.db.view(settings.COUCHDB_AUTH_VIEW, key = username.lower(), include_docs = True, limit = 1).rows
        
        if len(rows) and check_password(password, rows[0].value):
            return User(rows[0].doc, _db = self.db)
        pass

    



from datetime import datetime
from couchdbcurl.client import Document


class User(Document):

    def is_active(self):
        """Fake function. Always returns True"""
        return True
    
    def is_authenticated(self):
        """Links to self.is_superuser"""
        return '_id' in self
    
    def get_and_delete_messages(self):
        return None


    def has_module_perms(self, module):
        pass

    def save(self, *args, **kwargs):
        """Save user object and update ``last_login`` field"""
        
        if hasattr(self, 'backend'):
            backend = self.backend
            del(self.backend)
        else:
            backend = None
            
        if hasattr(self, 'last_login') and type(self.last_login) == datetime:
            from django.conf import settings
            self.last_login = self.last_login.strftime(settings.DATETIME_FMT)

        Document.save(self, *args, **kwargs)

        if backend:
            self.backend = backend

    def set_password(self, raw_password):
        """This is copy-paste from django.contrib.auth.models.User.set_password()"""
        
        import random
        from django.contrib.auth.models import get_hexdigest
        
        algo = 'sha1'
        salt = get_hexdigest(algo, str(random.random()), str(random.random()))[:5]
        hsh = get_hexdigest(algo, salt, raw_password)
        self.password = '%s$%s$%s' % (algo, salt, hsh)
        
        




import sys
from django.core.management.base import BaseCommand
from django.conf import settings
from django.contrib.auth.models import User as DjangoUser
from django_couch.auth.models import User

import django_couch
from couchdbcurl.client import ResourceConflict

class Command(BaseCommand):
    
    help = u'Creates admin user'
    
    def execute(self, db_key, username, *args, **kwargs):
        
        db = django_couch.db(db_key)
        skel = {
            'type': 'user',
            'is_staff': True,
            'is_superuser': True,
            'username': username,
        }
        
        u = User(skel, _db = db)
        password = DjangoUser.objects.make_random_password()
        u.set_password(password)
        #password_hash = u.password
        
        
        u.create('u_%s' % username)
        print 'User %s created. Document id: %s, password: %s' % (username, u.id, password)
        
        ## try:
        ##     db['u_%s' % username] = {"is_superuser": True, "is_staff": True, "last_login": None, "password": password_hash, "type": "user", "username": username}
        ##     print "User %s created. Password is: %s" % (username, password)
        ## except ResourceConflict:
        ##     print "Create user failed: conflict"
        
        
